@@ -42,10 +42,24 @@ def qdump__std____1__array(d, value):
     qdump__std__array(d, value)
 
 
+def qdump__std__function(d, value):
+    (ptr, dummy1, manager, invoker) = value.split('pppp')
+    if manager:
+        if ptr > 2:
+            d.putSymbolValue(ptr)
+        else:
+            d.putEmptyValue()
+        d.putBetterType(value.type)
+    else:
+        d.putValue('(null)')
+    d.putPlainChildren(value)
+
+
 def qdump__std__complex(d, value):
     innerType = value.type[0]
     (real, imag) = value.split('{%s}{%s}' % (innerType.name, innerType.name))
     d.putValue("(%s, %s)" % (real.display(), imag.display()))
+    d.putNumChild(2)
     if d.isExpanded():
         with Children(d, 2, childType=innerType):
             d.putSubItem("real", real)
@@ -887,10 +901,10 @@ def qedit__std__vector(d, value, data):
     import gdb
     values = data.split(',')
     n = len(values)
-    innerType = value.type[0]
+    innerType = value.type[0].name
     cmd = "set $d = (%s*)calloc(sizeof(%s)*%s,1)" % (innerType, innerType, n)
     gdb.execute(cmd)
-    cmd = "set {void*[3]}%s = {$d, $d+%s, $d+%s}" % (value.address, n, n)
+    cmd = "set {void*[3]}%s = {$d, $d+%s, $d+%s}" % (value.address(), n, n)
     gdb.execute(cmd)
     cmd = "set (%s[%d])*$d={%s}" % (innerType, n, data)
     gdb.execute(cmd)
@@ -964,6 +978,9 @@ def qdumpHelper__std__vector__QNX(d, value):
         else:
             d.putPlotData(start, size, innerType)
 
+def qform__std____1__vector():
+    return arrayForms()
+
 def qdump__std____1__vector(d, value):
     qdumpHelper__std__vector(d, value, True)
 
@@ -975,11 +992,23 @@ def qdump__std____debug__vector(d, value):
 
 
 def qedit__std__string(d, value, data):
-    d.call(value, "assign", '"%s"' % data.replace('"', '\\"'))
+    d.call('void', value, 'assign', '"%s"' % data.replace('"', '\\"'))
 
 def qedit__string(d, expr, value):
     qedit__std__string(d, expr, value)
 
+def qedit__std____cxx11__string(d, expr, value):
+    qedit__std__string(d, expr, value)
+
+def qedit__std__wstring(d, value, data):
+    d.call('void', value, 'assign', 'L"%s"' % data.replace('"', '\\"'))
+
+def qedit__wstring(d, expr, value):
+    qedit__std__wstring(d, expr, value)
+
+def qedit__std____cxx11__wstring(d, expr, value):
+    qedit__std__wstring(d, expr, value)
+
 def qdump__string(d, value):
     qdump__std__string(d, value)
 
@@ -1067,3 +1096,19 @@ def qdump__int8_t(d, value):
     d.putNumChild(0)
     d.putValue(value.integer())
 
+def qdump__std__byte(d, value):
+    d.putNumChild(0)
+    d.putValue(value.integer())
+
+def qdump__std__optional(d, value):
+    innerType = value.type[0]
+    (initialized, pad, payload) = d.split('b@{%s}' % innerType.name, value)
+    if initialized:
+        d.putItem(payload)
+        d.putBetterType(value.type)
+    else:
+        d.putSpecialValue("uninitialized")
+        d.putNumChild(0)
+
+def qdump__std__experimental__optional(d, value):
+    qdump__std__optional(d, value)
